"""
本作品采用知识共享署名-非商业性使用-相同方式共享 4.0 国际许可协议进行许可。
当您使用本作品相关图纸或源码等资料时，视为您同意并遵循此协议内容。

项目开源地址：https://www.laserblock.cn/thread-23573-1-1.html

作者：李嘉泳
邮箱：17620745509@126.COM
BiliBili：https://space.bilibili.com/106497171

Laserblock开源社区：https://www.laserblock.cn/
"""
# mPythonType:0
from mpython import *
import logging
import printer_info as info

# logging.basicConfig(level=logging.DEBUG)
logging.debug(
    "miniprinter micropython program v%s",
    info.info["device_info"]["version"]["overall"],
)
logging.info("program starting")
import printer_gui as gui
from servo import Servo
import time
import math
import printer_stepper as stepper
import printer_global_variable as glovar
import printer_gui_menu as menu
import printer_core as core
import os
import printer_file_system as file
import printer_gui as gui
import printer_network as net
import printer_input as inpt
import config as cfg
import printer_servo as srv

tl = logging.getLogger("main")

tl.debug("draw statup view")
gui.screen_fresh_percent(0)
gui.auto_show_text(1, "正 在 启 动")
gui.auto_show_text(3, "VER:" + info.info["device_info"]["version"]["overall"])

gui.screen_fresh_percent(20)

internal_files = {
    "摆臂校准测试": "calibrate_arm.bin",
    "打点校准测试": "calibrate_pen.bin",
    "雷宇科教爱上激光造物": "thunder.bin",
    "掌控板": "handPy.bin",
    "LASERBLOCK": "laserblock.bin",
}  # 内置的bin文件列表
tl.debug("internal files:%s", internal_files)


def update_user_file():
    tl.info("reading files:%s", glovar.bin_file_path)
    global file_list
    file_list = ["--------用户文件--------"]
    try:
        os.chdir(glovar.bin_file_path)  # 更改目录路径
    except OSError as errcode:
        tl.info("path:%s not found,creating", glovar.bin_file_path)
        os.mkdir(glovar.bin_file_path[1 : len(glovar.bin_file_path) - 1])
        os.chdir(glovar.bin_file_path)  # 再次更改目录路径
    os_listdir = os.listdir()
    tl.debug("file list:%s", os_listdir)
    os.chdir("/")  # 恢复目录路径
    tl.info("user file:")
    for file_name in os_listdir:  # 遍历目录路径
        tl.info(file_name)
        if file_name.endswith(".bin"):  # 只识别bin格式文件
            file_list.append(file_name)  # 目录列表追加当前文件
    file_list.extend(
        [
            "--------示例文件--------",
        ]
    )  # 目录列表追加后续功能
    for internal_files_name in internal_files:  # 遍历添加内置bin文件
        if internal_files_name == "摆臂校准测试" or internal_files_name == "打点校准测试":
            continue
        file_list.extend([internal_files_name])
    file_list.extend(["--------开发调试--------", "无线连接(Beta)", "摆臂校准测试", "打点校准测试", "关于"])
    tl.debug("final file list:%s", file_list)


update_user_file()  # 刷新文件内容

gui.screen_fresh_percent(50)


def backhome():
    tl.debug("backhome")
    menu.fresh_screen_preprocess(
        menu.current_list1, menu.current_list2, 0
    )  # 聚焦地址偏移0，返回原菜单原选项


gui.screen_fresh_percent(80)

# core.print_hex_file("/bin/China.bin")

if glovar.startup_connect:
    tl.info("startup network connecting")
    glovar.network_enabled = True  # 设置云连接标记位，开始刷新
    oled.fill(0)
    net.init()  # 连接网络

gui.screen_fresh_percent(99)

menu.switch_to_new_menu(file_list)  # 切换到主菜单

while True:
    if inpt.is_pressed(inpt.p):
        menu.fresh_screen_preprocess(
            menu.current_list1, menu.current_list2, -1
        )  # 列表内容不变，偏移-1
        continue
    if inpt.is_pressed(inpt.n):
        menu.fresh_screen_preprocess(
            menu.current_list1, menu.current_list2, 1
        )  # 列表内容不变，偏移1
        continue
    if inpt.is_pressed(inpt.a, True):
        tl.debug(
            "selected addr:%d,option:%s",
            menu.focus_address,
            menu.current_list1[menu.focus_address],
        )
        selected_option = menu.current_list1[menu.focus_address]  # 获取当前列表数组焦点位置内容为选中内容
        if selected_option[0] == "-":  # 不处理开头为-的标题内容
            tl.debug("title selected,not process")
            continue
        if selected_option.endswith(".bin"):  # 选中bin格式文件，打印
            tl.debug("bin file selected,start printing")
            core.print_hex_file(
                glovar.bin_file_path + selected_option
            )  # 打印文件，路径在/bin/下
            backhome()  # 打印完成后，返回原菜单原选项
            continue
        tl.debug("finding internal files array")
        for internal_files_name in internal_files:  # 遍历内置文件列表检查是否有对应选项
            if selected_option == internal_files_name:
                tl.debug(
                    "internal files found:%s,%s",
                    internal_files_name,
                    internal_files[internal_files_name],
                )
                core.print_hex_file(internal_files[internal_files_name])
                backhome()
                continue
        """
        下方为特殊选项
        """
        if selected_option == "无线连接(Beta)":
            tl.debug("local network connect selected")
            glovar.network_enabled = True  # 设置云连接标记位，开始刷新
            oled.fill(0)
            net.init()  # 连接网络
            backhome()
        if selected_option == "关于":
            tl.debug("about menu selected")
            about_list1 = [  # 新建关于菜单数组
                "----------关于----------",
                "名称",
                "作者",
                "邮箱",
                "",
                "BiliBili",
                "核心程序版本",
                "解析程序版本",
                "GUI版本",
                "网络程序版本",
            ]
            about_list2 = [
                "",
                info.info["device_info"]["name"],
                info.info["developer"]["name"],
                info.info["developer"]["mail"][:11],
                info.info["developer"]["mail"][11:],
                info.info["developer"]["bilibili"][27:],
                info.info["device_info"]["version"]["core"],
                info.info["device_info"]["version"]["analyze"],
                info.info["device_info"]["version"]["gui"],
                info.info["device_info"]["version"]["network"],
            ]
            menu.switch_to_new_menu(about_list1, about_list2)  # 切换到关于菜单
            continue  # 防误触
    if inpt.is_pressed(inpt.b, True):
        menu.switch_to_new_menu(file_list)  # 切换到主菜单
        continue
    if inpt.is_pressed(inpt.o):
        oled.fill(0)
        gui.auto_show_text(1, "正在走纸")
        while (
            inpt.is_pressed(inpt.o) or inpt.is_pressed(inpt.h)
        ) and not inpt.is_pressed(inpt.n):
            stepper.stepper.step()
        stepper.stepper.stop()  # 防止持续给某相输出电平
        backhome()
        continue
    if inpt.is_pressed(inpt.y):
        oled.fill(0)
        gui.auto_show_text(1, "正在退纸")
        while (
            inpt.is_pressed(inpt.y) or inpt.is_pressed(inpt.t)
        ) and not inpt.is_pressed(inpt.p):
            stepper.stepper.invert_step()
        stepper.stepper.stop()
        backhome()
        continue
    if glovar.network_enabled:  # 检查云连接标记位
        if net.update():  # 如果检查到下载文件
            update_user_file()  # 更新用户文件列表
            menu.switch_to_new_menu(file_list)  # 更新主菜单
