from mpython import *
import os
import printer_file_system as file
import math
import printer_input as inpt

myUI = UI(oled)
current_displaying_percent = 0

screen_x = 128
screen_y = 64


def get_screen_axis_y(number):
    if number < 0:
        return 0
    return number * 16


def get_screen_axis_x(this_str):
    str_len = 0
    this_str = str(this_str)
    for this_char in this_str:
        oc = ord(this_char)
        if oc < 255:
            if oc < 48 or (oc > 57 and oc < 65) or (oc > 90 and oc < 97):
                str_len += 4
                continue
            str_len += 8
        else:
            str_len += 12
    return str_len


def screen_fresh_percent(percent):
    global current_displaying_percent
    percent = round(percent)
    if current_displaying_percent == percent:
        return
    if percent == 100:
        current_displaying_percent = -1
    else:
        current_displaying_percent = percent
    progress_bar_two_side_interval = 24
    myUI.ProgressBar(
        progress_bar_two_side_interval,
        32,
        screen_x - 4 - (progress_bar_two_side_interval * 2),
        8,
        percent,
    )
    oled.show()


def screen_clear_line(line):
    oled.fill_rect(0, gui.get_screen_axis_y(line), screen_x, 16, 0)


def auto_show_text(line, str):
    text_two_side_interval = round(get_screen_axis_x(str) / 2)
    oled.DispChar(str, (screen_y - text_two_side_interval), get_screen_axis_y(line), 1)
    oled.show()


def view_hex_file(file_path):
    key_array = [inpt.a, inpt.b]
    if file.file.size > screen_x * file.file.height:
        loop_display_range = -1 * (file.file.width - screen_x)
        while not inpt.key_array_state(key_array):
            for i in range(0, loop_display_range, -1):
                oled.Bitmap(
                    i,
                    32,
                    file.file.byte_array,
                    file.file.width,
                    file.file.height,
                    1,
                )
                oled.show()
                if inpt.key_array_state(key_array):
                    return
            for i in range(loop_display_range, 0, 1):
                oled.Bitmap(
                    i,
                    32,
                    file.file.byte_array,
                    file.file.width,
                    file.file.height,
                    1,
                )
                oled.show()
                if inpt.key_array_state(key_array):
                    return
    else:
        oled.Bitmap(0, 32, file.file.byte_array, file.file.width, file.file.height, 1)
        oled.show()
        while not inpt.key_array_state(key_array):
            time.sleep_ms(50)


def print_content_confirm(file_path):
    oled.fill(0)
    auto_show_text(0, "开始打印？")
    view_hex_file(file_path)
    if inpt.is_pressed(inpt.a):
        return True
    return False


def convert_time(input_seconds):
    hour = 0
    minute = 0
    second = 0
    hour = math.floor(input_seconds / 3600)
    input_seconds = input_seconds - hour * 3600
    minute = math.floor(input_seconds / 60)
    input_seconds = input_seconds - minute * 60
    second = input_seconds
    return [hour, minute, second]


def handle_pause():
    if inpt.is_pressed(inpt.b):
        oled.fill(0)
        gui.auto_show_text(1, "任务已暂停")
        gui.auto_show_text(2, "确认取消吗")
        while True:
            if inpt.is_pressed_short(inpt.a):
                return True
            if inpt.is_pressed_short(inpt.b):
                oled.fill(0)
                gui.current_displaying_percent = -1
                gui.auto_show_text(1, "正  在  打  印")  # ---GUI---
                return False
            time.sleep_ms(50)
