from mpython import *
import urequests
import ujson
import config as cfg
import network
import utime
import os
import printer_gui as gui
import time
import printer_global_variable as glovar
import printer_core as core
import logging

tl = logging.getLogger("network")

seed = -1
status_file_url = ""
next_tick_time = 0
host_address = ""


def init():
    global status_file_url, host_address
    gui.auto_show_text(1, "正 在 连 接")
    gui.screen_fresh_percent(0)
    my_wifi = network.WLAN(network.STA_IF)
    """
    if my_wifi.isconnected():
        glovar.network_enabled = True
        gui.screen_fresh_percent(99)
        print("WiFi已经连接")
        return
    """
    tl.debug("init Wi-Fi")
    gui.screen_fresh_percent(25)
    tl.info("connecting:%s", cfg.cfg["network"]["name"])
    try:
        my_wifi.active(True)
        my_wifi.connect(cfg.cfg["network"]["name"], cfg.cfg["network"]["password"])
        loop_times = 0
        while not my_wifi.isconnected():
            if loop_times == 10:
                tl.info("retrying...")
                my_wifi.connect(
                    cfg.cfg["network"]["name"], cfg.cfg["network"]["password"]
                )
            if loop_times == 20:
                glovar.network_enabled = False
                tl.warning("cannot connect Wi-Fi")
                oled.fill(0)
                gui.auto_show_text(1, "连 接 失 败")
                gui.auto_show_text(2, "WiFi连接失败")
                time.sleep(3)
                return
            loop_times += 1
            time.sleep(0.5)
    except:
        glovar.network_enabled = False
        tl.warning("cannot connect Wi-Fi")
        oled.fill(0)
        gui.auto_show_text(1, "连 接 失 败")
        gui.auto_show_text(2, "WiFi连接失败")
        time.sleep(3)
        return
    tl.info("Wi-Fi connect successful")
    tl.info("ip address:%s", my_wifi.ifconfig()[0])
    gui.screen_fresh_percent(50)
    host_address = (
        "http://" + cfg.cfg["network"]["host"] + ":" + cfg.cfg["network"]["port"] + "/"
    )
    status_file_url = host_address + cfg.cfg["network"]["status_file_name"]
    tl.debug("status file path:%s", status_file_url)
    tl.debug("getting status file...")
    gui.screen_fresh_percent(75)
    try:
        urequests.get(status_file_url)
    except:
        tl.warning("cannot get status file!")
        oled.fill(0)
        gui.auto_show_text(1, "连 接 失 败")
        gui.auto_show_text(2, "无法获取状态")
        time.sleep(3)
        return
    tl.info("local network connect success")
    gui.screen_fresh_percent(99)


def download_files(path, file_name):
    oled.fill(0)
    gui.auto_show_text(1, "正 在 下 载")
    gui.screen_fresh_percent(0)
    tl.info("downloading:%s,path:%s", file_name, path)
    tl.debug("init download")
    bin_file = open(path + file_name, "wb")
    gui.screen_fresh_percent(25)
    tl.debug("downloading...")
    network_bin_file = urequests.get(host_address + file_name)
    gui.screen_fresh_percent(50)
    bin_file.write(network_bin_file.content)
    gui.screen_fresh_percent(75)
    bin_file.close()
    gui.screen_fresh_percent(99)
    tl.info("download complete!size:%s byte", os.stat(path + file_name)[6])


def update():
    global status_file_url, seed, next_tick_time
    if utime.ticks_ms() < next_tick_time:
        return
    next_tick_time = utime.ticks_ms() + cfg.cfg["network"]["fresh_time"]
    try:
        status_file = urequests.get(status_file_url)
    except:
        tl.debug("warning:cannot get status file")
        return False
    try:
        parsed_json = ujson.loads(status_file.content)
    except:
        tl.warning("cannot load status file")
        return False
    if seed == parsed_json["seed"]:
        return False
    tl.debug("catch update content")
    seed = parsed_json["seed"]
    if parsed_json["status"] == "standby":
        return False
    if parsed_json["status"] == "delete":
        oled.fill(0)
        gui.auto_show_text(1, "正 在 删 除")
        gui.screen_fresh_percent(0)
        tl.info(
            "catch delete command,filename:%s,path:%s",
            parsed_json["filename"],
            parsed_json["path"],
        )
        gui.screen_fresh_percent(50)
        os.chdir(parsed_json["path"])
        if parsed_json["filename"] == "*":
            for file_name in os.listdir():
                tl.debug("deleting:%s", file_name)
                try:
                    os.remove(file_name)
                except:
                    tl.warning("%s delete failed!", parsed_json["filename"])
        else:
            tl.debug("deleting:%s", parsed_json["filename"])
            try:
                os.remove(parsed_json["filename"])
            except:
                tl.warning("%s delete failed!", parsed_json["filename"])
        os.chdir("/")
        gui.screen_fresh_percent(99)
        return True
    if parsed_json["status"] == "download" or "print":
        if parsed_json["filename"][:1] == "[":
            tl.debug("catch file list")
            file_name = parsed_json["filename"][1 : len(parsed_json["filename"]) - 1]
            file_array = file_name.split(",")
            for this_file in file_array:
                download_files(parsed_json["path"], this_file)
        else:
            download_files(parsed_json["path"], parsed_json["filename"])
        if parsed_json["status"] == "print":
            core.print_hex_file(parsed_json["path"] + parsed_json["filename"])
        return True
    return False
