from mpython import *
import config as cfg
from servo import Servo
import logging
import time

tl = logging.getLogger("servo")


class servo_struct:
    def __init__(self, pin, max_angle, min_angle, steps_delay, angle_fix, steps=16):
        self.min_angle = min_angle
        self.max_angle = max_angle
        self.steps_delay = steps_delay
        self.current_step = 0
        self.angle = [self.min_angle for i in range(16)]
        self.steps = steps
        self.steps_per_angle = (self.max_angle - self.min_angle) / (self.steps - 1)
        self.angle_fix = angle_fix

        self.obj = Servo(pin, min_us=500, max_us=2500, actuation_range=180)


class arm_servo_struct(servo_struct):
    def __init__(
        self,
        pin,
        max_angle,
        min_angle,
        steps_delay,
        angle_fix,
        dead_zone_delay,
        dead_zone_angle,
        steps=16,
    ):
        servo_struct.__init__(
            self, pin, max_angle, min_angle, steps_delay, angle_fix, steps
        )
        self.is_in_dead_zone_fix_angle = False
        self.dead_zone_delay = dead_zone_delay
        self.dead_zone_angle = dead_zone_angle
        for check_precision_number in range(self.steps):
            self.angle[check_precision_number] = (
                round(self.min_angle + check_precision_number * self.steps_per_angle)
            ) + self.angle_fix[check_precision_number]

    def move(self, step, delay=True):
        if step < 0 or step > 15:
            step = 0
            self.current_step = 15
        if self.current_step == step and not self.is_in_dead_zone_fix_angle:
            return
        if step > self.current_step:
            self.obj.write_angle(self.angle[step])
        elif step == self.current_step:
            self.obj.write_angle(self.angle[step])
            self.is_in_dead_zone_fix_angle = False
        else:
            self.obj.write_angle(self.angle[step] - self.dead_zone_angle)
            self.is_in_dead_zone_fix_angle = True
        servo_move_steps = self.current_step - step
        if servo_move_steps < 0:
            servo_move_steps *= -1
        if delay:
            time.sleep_ms(self.steps_delay * servo_move_steps)
        self.current_step = step


class dot_servo_struct(servo_struct):
    def __init__(self, pin, max_angle, min_angle, steps_delay, angle_fix, steps=16):
        servo_struct.__init__(
            self, pin, max_angle, min_angle, steps_delay, angle_fix, steps
        )
        self.status = False
        for check_precision_number in range(self.steps):
            self.angle[check_precision_number] = (
                self.max_angle + self.angle_fix[check_precision_number]
            )

    def dot(self, is_dot, step=0):
        if not is_dot:
            self.stetus = False
            self.obj.write_angle(self.min_angle)
            return
        self.statue = True
        self.obj.write_angle(self.angle[step])

    def dot_once(self, step=0):
        self.dot(True, step)
        time.sleep_ms(self.steps_delay)
        self.dot(False, step)
        time.sleep_ms(self.steps_delay)

    def invert(self, status, step=0):
        if status == self.status:
            return
        if status:
            self.dot(True, step)
            time.sleep_ms(self.steps_delay)
        else:
            self.dot(False, step)
            time.sleep_ms(self.steps_delay)


arm = arm_servo_struct(
    pin=cfg.cfg["hardware"]["arm_servo_pin"],
    min_angle=cfg.cfg["core"]["arm_servo"]["min_angle"],
    max_angle=cfg.cfg["core"]["arm_servo"]["max_angle"],
    steps_delay=cfg.cfg["core"]["arm_servo"]["delay"],
    angle_fix=cfg.cfg["core"]["arm_servo"]["angle_fix"],
    dead_zone_delay=0,
    dead_zone_angle=cfg.cfg["core"]["arm_servo"]["dead_zone"],
)
arm.move(-1)
tl.debug("arm servo info:%s", arm.__dict__)

dot = dot_servo_struct(
    pin=cfg.cfg["hardware"]["dot_servo_pin"],
    min_angle=cfg.cfg["core"]["dot_servo"]["min_angle"],
    max_angle=cfg.cfg["core"]["dot_servo"]["max_angle"],
    steps_delay=cfg.cfg["core"]["dot_servo"]["delay"],
    angle_fix=cfg.cfg["core"]["dot_servo"]["angle_fix"],
)
dot.dot(False, 0)
tl.debug("dot servo info:%s", dot.__dict__)
