# mPythonType:0
from mpython import *
import config as cfg
import logging

tl = logging.getLogger("stepper")


class stepper_struct:
    def __init__(self, pin, step_pwm, delay):
        self.pin_obj = [None for i in range(4)]
        for i in range(4):
            self.pin_obj[i] = MPythonPin(pin[i], PinMode.OUT)
        self.step_pwm_count = step_pwm
        self.delay = delay

    def set_gpio(self, pin_state):
        for i in range(4):
            self.pin_obj[i].write_digital(pin_state[i])
        time.sleep_us(self.delay)

    def step(self):
        self.set_gpio([1, 0, 0, 0])
        self.set_gpio([1, 1, 0, 0])
        self.set_gpio([0, 1, 0, 0])
        self.set_gpio([0, 1, 1, 0])
        self.set_gpio([0, 0, 1, 0])
        self.set_gpio([0, 0, 1, 1])
        self.set_gpio([0, 0, 0, 1])
        self.set_gpio([1, 0, 0, 1])

    def invert_step(self):
        self.set_gpio([1, 0, 0, 0])
        self.set_gpio([1, 0, 0, 1])
        self.set_gpio([0, 0, 0, 1])
        self.set_gpio([0, 0, 1, 1])
        self.set_gpio([0, 0, 1, 0])
        self.set_gpio([0, 1, 1, 0])
        self.set_gpio([0, 1, 0, 0])
        self.set_gpio([1, 1, 0, 0])

    def stop(self):
        self.set_gpio([0, 0, 0, 0])

    def steps(self, steps=0):
        # tl=logging.getLogger('stepper steps:%d',steps)
        if steps == 0:
            steps = self.step_pwm_count
        if steps > 0:
            for count in range(int(steps)):
                self.step()
        if steps < 0:
            steps = steps * -1
            for count in range(int(steps)):
                self.invert_step()
        self.stop()

    def fast_mode(self):
        tl.debug("fast mode set")
        self.delay = cfg.cfg["core"]["stepper"]["fastmode_delay"]

    def normal_mode(self):
        tl.debug("normal mode set")
        self.delay = cfg.cfg["core"]["stepper"]["delay"]


stepper = stepper_struct(
    pin=[
        cfg.cfg["hardware"]["stepper_pin_1"],
        cfg.cfg["hardware"]["stepper_pin_2"],
        cfg.cfg["hardware"]["stepper_pin_3"],
        cfg.cfg["hardware"]["stepper_pin_4"],
    ],
    step_pwm=cfg.cfg["core"]["stepper"]["step_pwm"],
    delay=cfg.cfg["core"]["stepper"]["delay"],
)
stepper.fast_mode()
stepper.stop()
tl.debug("stepper info:%s", stepper.__dict__)
